#!/bin/bash
color_bit_depth="$1"
worksheet_name="$2"
target_range_address_left_top="$3"
target_range_address_right_bottom="$4"

if [[ -z "$color_bit_depth" || -z "$worksheet_name" || -z "$target_range_address_left_top" || -z "$target_range_address_right_bottom" ]]
then
	echo "第一个参数为【颜色位深】，其值不能为空。其数值需要是 3 的倍数，且不能超过 24。" >&2
	echo "第二个参数为【工作表的名称】，其值不能为空。如：工作表1、Sheet1。" >&2
	echo "第三个参数为【条件格式目标区域的左上角单元格的地址】，其值不能为空。如：A1。" >&2
	echo "第四个参数为【条件格式目标区域的右下角单元格的地址】，其值不能为空。如：C20" >&2
	exit 4
fi
mod="$(( $color_bit_depth % 3 ))"
if [[ ! "$mod" -eq 0 ]]
then
	echo "【颜色位深】数值需要是 3 的倍数。" >&2
	exit 4
fi
if [[ "$color_bit_depth" -gt 24 ]]
then
	echo "【颜色位深】数值不能超过 24。" >&2
	exit 4
fi

color_count=$(( 2 ** "$color_bit_depth" ))
echo "将会按 $color_count 个颜色生成 content.xml 中的条件格式样式 xml 数据…" >&2

rgb_single_color_bit_depth=$(( $color_bit_depth / 3 ))
echo "单色位深 = $rgb_single_color_bit_depth" >&2
rgb_single_color_count=$(( 2 ** $rgb_single_color_bit_depth ))
rgb_single_color_mask=$(( $rgb_single_color_count - 1 ))
echo "单色数量 = $rgb_single_color_count" >&2

echo '下面生成的 <style:style></style:style>，需要放到 content.xml 内容的 <office:automatic-styles></office:automatic-styles> 内' >&2

echo '<style:style style:name="ce2" style:family="table-cell" style:parent-style-name="Default">'
echo '	<style:table-cell-properties style:text-align-source="fix" style:repeat-content="false" fo:border="none" style:vertical-align="middle"/>'
echo '	<style:paragraph-properties fo:text-align="center" fo:margin-left="0cm"/>'
echo '	<style:text-properties fo:font-size="6pt" style:font-size-asian="6pt" style:font-size-complex="6pt"/>'
for ((i=0; i<$color_count; i++))
do
	r=$(( $i >> $rgb_single_color_bit_depth >> $rgb_single_color_bit_depth ))
	g=$(( ($i >> $rgb_single_color_bit_depth) & $rgb_single_color_mask ))
	b=$(( $i & $rgb_single_color_mask ))
	r_8bit=$(( 255 * $r / $rgb_single_color_mask ))
	g_8bit=$(( 255 * $g / $rgb_single_color_mask ))
	b_8bit=$(( 255 * $b / $rgb_single_color_mask ))
	cell_value_as_index_to_color_hex=$( printf "%02x%02x%02x" $r_8bit $g_8bit $b_8bit )
	# cfsi: Conditional Formatting Style Index
	echo '	<style:map style:condition="cell-content()='${i}'" style:apply-style-name="cfsi'${cell_value_as_index_to_color_hex}'" style:base-cell-address="'${worksheet_name}'.'${target_range_address_left_top}'"/>'
done
echo '</style:style>'


echo
echo
echo
echo

echo '下面生成的 <calcext:conditional-formats></calcext:conditional-formats>，需要放到 content.xml 内容的 <table:table> </table:table> 内' >&2

echo '<calcext:conditional-formats>'
echo '	<calcext:conditional-format calcext:target-range-address="'${worksheet_name}'.'${target_range_address_left_top}':'${worksheet_name}'.'${target_range_address_right_bottom}'">'
for ((i=0; i<$color_count; i++))
do
	r=$(( $i >> $rgb_single_color_bit_depth >> $rgb_single_color_bit_depth ))
	g=$(( ($i >> $rgb_single_color_bit_depth) & $rgb_single_color_mask ))
	b=$(( $i & $rgb_single_color_mask ))
	r_8bit=$(( 255 * $r / $rgb_single_color_mask ))
	g_8bit=$(( 255 * $g / $rgb_single_color_mask ))
	b_8bit=$(( 255 * $b / $rgb_single_color_mask ))
	cell_value_as_index_to_color_hex=$( printf "%02x%02x%02x" $r_8bit $g_8bit $b_8bit )
	# cfsi: Conditional Formatting Style Index
	echo '		<calcext:condition calcext:value="='${i}'" calcext:apply-style-name="cfsi'${cell_value_as_index_to_color_hex}'" calcext:base-cell-address="'${worksheet_name}'.'${target_range_address_left_top}'"/>'
done
echo '	</calcext:conditional-format>'
echo '</calcext:conditional-formats>'
